// SampleDisplay.h : Declaration of the CSampleDisplay

#ifndef __SAMPLEDISPLAY_H_
#define __SAMPLEDISPLAY_H_

#include "resource.h"       // main symbols
#include <atlctl.h>

#include <math.h>

// OpenGL include files..
#include "gl/gl.h"
#include "gl/glaux.h"  

// Ogl helper functions..
#include "OpenGLHelperTemplates.h"

// OpenGL libraries to link
#pragma comment (lib, "opengl32.lib")
#pragma comment (lib, "glu32.lib")
//#pragma comment (lib, "glaux.lib") Could use this if needed..




/////////////////////////////////////////////////////////////////////////////
// CSampleDisplay
class ATL_NO_VTABLE CSampleDisplay : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<ISampleDisplay, &IID_ISampleDisplay, &LIBID_SAMPLEAXOPENGLCONTROLVCLib>,
	public CComControl<CSampleDisplay>,
	public IPersistStreamInitImpl<CSampleDisplay>,
	public IOleControlImpl<CSampleDisplay>,
	public IOleObjectImpl<CSampleDisplay>,
	public IOleInPlaceActiveObjectImpl<CSampleDisplay>,
	public IViewObjectExImpl<CSampleDisplay>,
	public IOleInPlaceObjectWindowlessImpl<CSampleDisplay>,
	public IPersistStorageImpl<CSampleDisplay>,
	public ISpecifyPropertyPagesImpl<CSampleDisplay>,
	public IQuickActivateImpl<CSampleDisplay>,
	public IDataObjectImpl<CSampleDisplay>,
	public IProvideClassInfo2Impl<&CLSID_SampleDisplay, NULL, &LIBID_SAMPLEAXOPENGLCONTROLVCLib>,
	public CComCoClass<CSampleDisplay, &CLSID_SampleDisplay>,

  public IPersistPropertyBagImpl<CSampleDisplay>
{
public:
	CSampleDisplay()
	{
    // For an OpenGL display, we must be "windowed only". 
    // The window will be used to create an OpenGL drawing context later in "OnCreate" (WM_CREATE handler)
		m_bWindowOnly = TRUE;  // Must be set to attach OpenGL context...
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SAMPLEDISPLAY)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSampleDisplay)
	COM_INTERFACE_ENTRY(ISampleDisplay)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)

  COM_INTERFACE_ENTRY(IPersistPropertyBag)

END_COM_MAP()

BEGIN_PROP_MAP(CSampleDisplay)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)

  PROP_ENTRY( "SkyColor",     4,  CLSID_NULL)
  PROP_ENTRY( "GroundColor",  5,  CLSID_NULL)
  PROP_ENTRY( "ShowHeading",  6,  CLSID_NULL)

END_PROP_MAP()

BEGIN_MSG_MAP(CSampleDisplay)
	CHAIN_MSG_MAP(CComControl<CSampleDisplay>)
	DEFAULT_REFLECTION_HANDLER()

  MESSAGE_HANDLER(WM_CREATE,      OnCreate)       // Need this to create our OpenGL drawing context.. Add by hand or wizard
  MESSAGE_HANDLER(WM_DESTROY,     OnDestroy)      // Need this to destroy our OpenGL drawing context.. Add by hand or wizard
  MESSAGE_HANDLER(WM_ERASEBKGND,  OnEraseBkgnd)   // Need this to eliminate redundant background erasing...prevents windows generated flicker
  MESSAGE_HANDLER(WM_SIZE,        OnSize)         // Need this to ensure "glViewport" is set properly on resize

END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// ISampleDisplay
public:

  // Member variables for OpenGL
  HDC                   mHDC;
  HGLRC                 mGraphicsContextId; 
  PIXELFORMATDESCRIPTOR mPfd;

  // Control's member variables
  double                mPitchAngle;        
  double                mRollAngle;  
  double                mHeading;
  VARIANT_BOOL          mShowHeading;
  OLE_COLOR             mSkyColor;
  OLE_COLOR             mGroundColor;

  double                mXOrigin;
  double                mYOrigin;


  
  STDMETHOD(FinalConstruct)( void )
  {
    // Add constructor code here..

    // Initialize all of our member variables
    mRollAngle = 0.0;
    mPitchAngle = 0.0;
    mHeading = 0.0;
    mShowHeading = VARIANT_TRUE;
    mSkyColor = RGB (10, 10, 255);
    mGroundColor = RGB (141, 97, 24);
    
    mYOrigin = 0.0;
    mXOrigin = 0.0;
    
    // OpenGL and Windows stuff..
    mGraphicsContextId = NULL;
    mHDC = NULL;

    return S_OK;
  }

  STDMETHOD(FinalRelease)( void )
  {
    // Add additional destuction code here..
    return S_OK;
  }


  // This is the "Heart" of the drawing code... Put all drawing code here...
  HRESULT OnDraw( ATL_DRAWINFO& di )
	{
    // If the OpenGL context is not initialized yet, don't draw or it may crash...
    if ( !mGraphicsContextId )
    {
      return S_OK;
    }

    // Make our OpenGL "context" the current one (Must do this)
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    // Clear out the background 
    glClear( GL_COLOR_BUFFER_BIT );

    drawMovingAreas();
    drawFixedAreas();
    if ( mShowHeading )
    {
      drawTapeHeading();
    }

    // Output our rending to the screen
    ::SwapBuffers( mHDC );
    return S_OK;
	}


  // The following methods are "Event Handlers" for windows events. In order to have the methods called, we must first
  // add a handler to the "BEGIN_MSG_MAP" area of the code above. You can do this using the wizard or by hand..
  
  // This method gets called after our "Window" is created... At this time, it's ok to create the OpenGL drawing context
  virtual LRESULT OnCreate( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
  {
    // Get our "DC" (DeviceContext), we'll need this to "SwapBuffers" later...
    mHDC = this->GetDC();

    BOOL isHardware = TRUE;    // Always default to hardware unless shut off...
    BOOL verticalSync = FALSE; // Wait for vertical sync? No.. It runs faster and we have seen no "tearing" so far..
    BOOL doubleBuffer = TRUE;  // Always default to double buffered to reduce tearing

    // Call the helper functions to setup the OpenGL drawing context
    tSetupOGLPixelFormat( mHDC, isHardware, doubleBuffer, verticalSync, 16, 32, &mPfd );

    mGraphicsContextId = wglCreateContext( mHDC );
    if ( mGraphicsContextId == NULL ) return E_FAIL;
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    // Create call lists so we can output text...Don't use this example for your code... See OpenGlHelperTemplates.h
    tGenerateNumberCallLists();

    // Set the clear color
    glClearColor( 0.70f, 0.70f, 0.70f, 1.0f );  

    // Set the ortho... Doesn't change for this display
    glOrtho( -135.0, 135.0, -135.0, 135.0, -1.0,  1.0 );

    return S_OK;
  }

  // This method gets called just before our "Window" is being destroyed... Clean up the OpenGL drawing context here..
  virtual LRESULT OnDestroy( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
  {
    ::wglMakeCurrent( NULL, NULL );  // Don't forget this!! Must make sure that we're not the current context before destruction
  
    ReleaseDC( mHDC );  // Release our DC.. Windows cleanup..
  
    // Delete our OpenGL drawing context..
    if ( mGraphicsContextId )
    {
      ::wglDeleteContext( mGraphicsContextId );
      mGraphicsContextId = NULL;
    }

    return 0;  // Let windows do its final cleanup
  }

  // Flicker prevention..
  virtual LRESULT OnEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
  {
    // By returning 1 and "bHandled=TRUE" we are telling windows NOT to erase our background. This will prevent
    // a redundant "EraseBackground" operation that will produce an obnoxious "flicker" on the screen. 
    // We'll let the OpenGL window do all erasing/drawing from now on.... The "WM_PAINT" message will eventually
    // call our "OnDraw" method above. That's were all of the OpenGL rending code will be called..
    bHandled = TRUE;
    return 1;
  }

    // Set the OpenGL viewport properly every time someone resizes us...
  virtual LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
  {
    if ( !mGraphicsContextId || !mHDC ) return 0;

    RECT rc;
    GetClientRect( &rc );
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    ::glViewport( 0,0, rc.right - rc.left, rc.bottom - rc.top );   
  
    this->FireViewChange();
    return 0;
  }

  // Every "Property" has a get_ and a put_ method here. The wizard puts the code stubs here and in the .cpp
  // For this example, I've added all of the code to the .hpp file and removed the wizard stubs from the .cpp

	STDMETHOD(get_Roll)(/*[out, retval]*/ double *pVal)
  {
    if( !pVal ) return E_POINTER; 
    *pVal = mRollAngle;          
    return S_OK;
  }
  STDMETHOD(put_Roll)(/*[in]*/ double newVal)
  {
    if ( newVal == mRollAngle ) return S_OK; 
    mRollAngle = newVal;             
    this->m_bRequiresSave = TRUE;   
    this->FireViewChange();         
    return S_OK;
  }

  STDMETHOD(get_Pitch)(/*[out, retval]*/ double *pVal)
  {
    if( !pVal ) return E_POINTER;  
    *pVal = mPitchAngle;           
    return S_OK;
  }
	STDMETHOD(put_Pitch)(/*[in]*/ double newVal)
  {
    if ( newVal == mPitchAngle ) return S_OK; 
    mPitchAngle = newVal;             
    this->m_bRequiresSave = TRUE;   
    this->FireViewChange();         
    return S_OK;
  }

  STDMETHOD(get_Heading)(/*[out, retval]*/ double *pVal)
  {
    if( !pVal ) return E_POINTER;  
    *pVal = mHeading;              
    return S_OK;
  }
  STDMETHOD(put_Heading)(/*[in]*/ double newVal)
  {
    if ( newVal == mHeading ) return S_OK; 
    mHeading = newVal;             
    this->m_bRequiresSave = TRUE;   
    this->FireViewChange();         
    return S_OK;
  }
	
	STDMETHOD(get_GroundColor)(/*[out, retval]*/ OLE_COLOR *pVal)
  {
    if( !pVal ) return E_POINTER;   
    *pVal = mGroundColor;           
    return S_OK;
  }
  STDMETHOD(put_GroundColor)(/*[in]*/ OLE_COLOR newVal)
  {
    if ( newVal == mGroundColor ) return S_OK;  
    mGroundColor = newVal;           
    this->m_bRequiresSave = TRUE;    
    this->FireViewChange();        
    return S_OK;
  }

	STDMETHOD(get_SkyColor)(/*[out, retval]*/ OLE_COLOR *pVal)
  {
    if( !pVal ) return E_POINTER;  
    *pVal = mSkyColor;             
    return S_OK;
  }
	STDMETHOD(put_SkyColor)(/*[in]*/ OLE_COLOR newVal)
  {
    if ( newVal == mSkyColor ) return S_OK; 
    mSkyColor = newVal;             
    this->m_bRequiresSave = TRUE;   
    this->FireViewChange();         
    return S_OK;
  }

  STDMETHOD(get_ShowHeading)(/*[out, retval]*/ VARIANT_BOOL *pVal)
  {
    if( !pVal ) return E_POINTER;  // If bad value passed in, return error
    *pVal = mShowHeading;          // Assign the return value to our member variable contents
    return S_OK;
  }
	STDMETHOD(put_ShowHeading)(/*[in]*/ VARIANT_BOOL newVal)
  {
    if ( newVal == mShowHeading ) return S_OK; // Nothing changed, just return
    mShowHeading = newVal;          // Assign the new value to our member variable
    this->m_bRequiresSave = TRUE;   // This tells your parent "the ActiveX Container" that you require saving because something changed
    this->FireViewChange();         // This causes the display to be redrawn with the new setting
    return S_OK;
  }
  
  
  // OpenGL drawing code...
  #define SHOW_ME 20
  #define LINE_LONG 25
  #define LINE_SHORT 15

  void drawMovingAreas()
  {
    double Scaled_Pitch_Angle;
    int loop, Pitch_Ladder;
    int xl, xr, y;
    int heading_chars;
    char heading_string[15];
  
    //-----------------------------------------------------
    // SAVE THE TRANSFORMATION MATRIX THEN ROTATE HORIZON 
    //-----------------------------------------------------
    glPushMatrix();
    glRotatef( (float)mRollAngle, 0.0f, 0.0f, 1.0f );
  
    //----------
    // DRAW SKY 
    //----------
    tSetColor( mSkyColor );
  
    Scaled_Pitch_Angle = mPitchAngle*4;
    glBegin( GL_POLYGON );
    {
        glVertex2d(-165, mYOrigin-(int)(Scaled_Pitch_Angle)); 
        glVertex2d(-165 , +165 ); 
        glVertex2d(165 , +165 ); 
        glVertex2d(165 , mYOrigin-(int)(Scaled_Pitch_Angle)); 
    }
    glEnd();
  
    //-------------
    // DRAW GROUND
    //-------------
    tSetColor( mGroundColor );
    glBegin( GL_POLYGON );
    {
        glVertex2d(-165, -165); 
        glVertex2d(-165 , mYOrigin-(int)(Scaled_Pitch_Angle) ); 
        glVertex2d(165 , mYOrigin-(int)(Scaled_Pitch_Angle) ); 
        glVertex2d(165 , -165); 
    }
    glEnd();
  
    //---------------------------
    // DRAW PITCH ANGLE MARKERS
    //---------------------------
    glColor3d(0.6, 0.6, 0.6);
    //loop = (int)mPitchAngle/10;
    loop = (int)(fmod(mPitchAngle,360.0)/10);
    loop *= 10;
    glLineWidth( (GLfloat)2 );
    //glEnable( GL_LINE_SMOOTH );
  
    Pitch_Ladder = loop-SHOW_ME;
    while ( Pitch_Ladder <= loop+SHOW_ME )
    {
      if (Pitch_Ladder != 0)
      {
        glBegin( GL_LINES );
        {
          xl = (int)mXOrigin-LINE_LONG;
          xr = (int)mXOrigin+LINE_LONG;
          y = (int)mYOrigin+(Pitch_Ladder*4)-(int)(Scaled_Pitch_Angle);
          glVertex2i( xl, y );
          glVertex2i( xr, y );
        }
        glEnd();
        
        sprintf( heading_string, "%d", Pitch_Ladder );
        int pLen = strlen(heading_string); pLen = pLen;
        if (Pitch_Ladder >= 0)
        {
          glPushMatrix();
          {
            glTranslatef((float)xr+4,(float)y-4, (float)0);
            glColor3d(0.7,0.7,0.7);
            tPrintStrokedString(heading_string);
            glColor3d(0.6, 0.6, 0.6);
            //glEnable( GL_LINE_SMOOTH );
          }
          glPopMatrix();
        }
        if (Pitch_Ladder <= 0)
        {
          heading_chars = strlen(heading_string) - 1;
          glPushMatrix();
          // Center Neg characters 
          glTranslatef((float)xl-(heading_chars*8)-11, (float)y-4,0);
          
          glColor3d(0.7,0.7,0.7);
          tPrintStrokedString(heading_string);
          glColor3d(0.6, 0.6, 0.6);
          //glEnable( GL_LINE_SMOOTH );
          glPopMatrix();
        }
      }
      
      glBegin( GL_LINES );
      {
        xl = (int)mXOrigin-LINE_SHORT;
        xr = (int)mXOrigin+LINE_SHORT;
        y = (int)mYOrigin+((Pitch_Ladder+5)*4)-(int)(Scaled_Pitch_Angle);
        glVertex2i( xl, y );
        glVertex2i( xr, y );
      }
      glEnd(); 
      
      Pitch_Ladder+=10;
    }
    glLineWidth( (GLfloat)1 );
  
    //glDisable( GL_LINE_SMOOTH );
  
    //-------------------------------
    // RESTORE TRANSFORMATION MATRIX 
    //-------------------------------
    glPopMatrix(); 
  }


  void drawFixedAreas()
  {
    int loop;
    const GLint corner_points[12][2] = 
    { {-130,  130},   // UL
      {-130,   99},
      { -99,  130},
      { 130,  130},   // UR
      {  99,  130},
      { 130,   99},
      { 130, -130},   // LL
      { 130,  -99},
      {  99, -130},
      {-130, -130},   // LR
      { -99, -130},
      {-130,  -99} };
    
    
    /************************************************
    * MASK OUT AREA AROUND ARTIFICIAL HORIZON AREA *
    ************************************************/
    glColor3d(0.6, 0.6, 0.6);
    glRecti(-140,-140,-130,140);
    glRecti(-140,140,140,130);
    glRecti(130,140,140,-140);
    glRecti(-140,-130,140,-140);
    
    glRecti(-140,140,140,180);
    
    glBegin( GL_POLYGON );
      glVertex2iv(corner_points[0]);
      glVertex2iv(corner_points[1]);
      glVertex2iv(corner_points[2]);
    glEnd();
    glBegin( GL_POLYGON );
      glVertex2iv(corner_points[3]);
      glVertex2iv(corner_points[4]);
      glVertex2iv(corner_points[5]);
    glEnd();
    glBegin( GL_POLYGON );
      glVertex2iv(corner_points[6]);
      glVertex2iv(corner_points[7]);
      glVertex2iv(corner_points[8]);
    glEnd();
    glBegin( GL_POLYGON );
      glVertex2iv(corner_points[9]);
      glVertex2iv(corner_points[10]);
      glVertex2iv(corner_points[11]);
    glEnd();
    
    //glEnable( GL_LINE_SMOOTH );
    
    glBegin( GL_LINE_LOOP );
      glColor3d( 0.0, 0.0, 0.0 );
      glVertex2iv(corner_points[11]);
      glVertex2iv(corner_points[1]);
      glVertex2iv(corner_points[2]);
      glVertex2iv(corner_points[4]);
      glColor3d( 1.0, 1.0, 1.0 );
      glVertex2iv(corner_points[5]);
      glVertex2iv(corner_points[7]);
      glVertex2iv(corner_points[8]);
      glVertex2iv(corner_points[10]);
    glEnd();
    
    
    //-----------------------------------
    // Draw the tiny airplane
    glColor3d(1.0, 1.0, 0.0);
    glLineWidth( (GLfloat)2 );
    
    glBegin( GL_LINES );
    {
      glVertex2i(-50, 0);
      glVertex2i(-25, 0);
    }
    glEnd();
    
    glBegin( GL_LINES );
    {
      glVertex2i(25, 0);
      glVertex2i(50, 0);
    }
    glEnd();
    
    glBegin( GL_LINE_STRIP );
    {
      glVertex2i(-25,0);
      glVertex2i(0,-23);
      glVertex2i((int)mXOrigin, (int)mYOrigin);
      glVertex2i(0,-23);
      glVertex2i(25,0);
    }
    glEnd();
    
    glLineWidth( (GLfloat)1 );
    
    //glDisable( GL_LINE_SMOOTH );
    //-----------------------------------
    
    
    // Draw circular tick marks 
    glPushMatrix();
    glColor3d(1.0, 1.0, 1.0);
    loop = -10;
    int outer; int inner;
    while ( loop < 350 )
    {
      if (loop%15 == 0)
      {
        outer = 120;
        inner = 106;
      }
      else
      {
        outer = 120;
        inner = 115;
      }
      
      glRotatef( 10, 0.0, 0.0, 1.0);
      if (loop < 165)
      {
        glBegin( GL_LINE_STRIP );
        {
          glVertex2i(0,outer);
          glVertex2i(0,inner);
        }
        glEnd(); 
      }
      loop += 5;
    }
    glPopMatrix();

    /******************************************************
    * DRAW 3D LINES AROUND HORIZON AND TAPE HEADING AREA *
    ******************************************************/
    glColor3d( 1.0, 1.0, 1.0 );
    glBegin( GL_LINE_STRIP );
      glVertex2i(-139,-139);
      glVertex2i(-139,179);
      glVertex2i(140,179);
    glEnd();
    glBegin( GL_LINE_STRIP );
      glVertex2i(-139,140);
      glVertex2i(140,140);
    glEnd();
    
    glColor3d( 0.0, 0.0, 0.0 );
    glBegin( GL_LINE_STRIP );
      glVertex2i(140,179);
      glVertex2i(140,-140);
      glVertex2i(-139,-140);
    glEnd();
    glBegin( GL_LINE_STRIP );
      glVertex2i(-139,142);
      glVertex2i(140,142);
    glEnd();
    
  }

  void drawTapeHeading()
  {
    //----------------------------
    glColor3d(0.6, 0.6, 0.6);
    glRecti(-75,130,75,95);
  
    glColor3d( 1.0, 1.0, 1.0 );
    glBegin( GL_LINE_STRIP );
      glVertex2i(-75, 130);
      glVertex2i(-75, 95);
    glEnd();
  
    glColor3d( 0.0, 0.0, 0.0 );
    glBegin( GL_LINE_STRIP );
      glVertex2i(-75, 95);
      glVertex2i(75, 95);
    glEnd();
  
    glBegin( GL_LINE_STRIP );
      glVertex2i(75, 95);
      glVertex2i(75, 130);
    glEnd();
    //------------------------------
  
    //    testHeading  += 1;
    //    mHeading = testHeading;
    int offset = -50;
  
    int loop,
      pixel_counter,
      temp_heading,
      heading_check;
    char heading_string[5];
  
    glColor3d( 1.0, 0.0, 0.0 );
    glLineWidth( 2 );
    glBegin( GL_LINE_STRIP );
      glVertex2f(0,139); 
      glVertex2f(0,125); 
    glEnd();
    glLineWidth( 1 );
  
    //changed in the control room.  Let me know if we need to add this for the control to work properly.
    glColor3d( 0.0, 0.0, 0.0 );
    temp_heading = (int)(mHeading+0.5);
  
    // ** Tick marks and values to the right of the heading **
    pixel_counter = 0;
    loop = 0;
    while ( loop < 13 )
    {
      if (temp_heading < 0) temp_heading = (int)(fmod(temp_heading, 360.0));
      if (temp_heading > 359) temp_heading = (int)(fmod(temp_heading, 360.0));
      if (temp_heading < 0) temp_heading += 360;
      if (temp_heading > 359) temp_heading -= 360;
      heading_check = abs(temp_heading - ((int)(temp_heading/10)*10));
      if(heading_check == 0)
      {
        glBegin( GL_LINE_STRIP );
        glVertex2i(pixel_counter,147+offset);
        glVertex2i(pixel_counter,157+offset);
        glEnd();
        sprintf(heading_string,"%3.3d",temp_heading);
        glPushMatrix();
        glTranslatef(pixel_counter-13.0f,162.0f+offset,0.0f);
        glLineWidth( 2 );
        tPrintStrokedString(heading_string);
        glLineWidth( 1 );
        glPopMatrix();
      }
    
      if(heading_check == 2 || heading_check == 4 ||
        heading_check == 6 || heading_check == 8)
      {
        glBegin( GL_LINE_STRIP );
        glVertex2i(pixel_counter,147+offset);
        glVertex2i(pixel_counter,152+offset);
        glEnd();
      }
      temp_heading++;
      pixel_counter += 5;
      loop++;
    }
  
    // ** Tick marks and values to the left of the heading **
    temp_heading = (int)(mHeading + 0.5);
  
    pixel_counter = 0;
    loop = 0;
    while ( loop < 13 )
    {
      if (temp_heading < 0) temp_heading = (int)(fmod(temp_heading, 360.0));
      if (temp_heading > 359) temp_heading = (int)(fmod(temp_heading, 360.0));
      if (temp_heading < 0) temp_heading += 360;
      if (temp_heading > 359) temp_heading -= 360;
      heading_check = abs(temp_heading - ((int)(temp_heading/10)*10));
      if(heading_check == 0)
      {
        glBegin( GL_LINE_STRIP );
        glVertex2i(pixel_counter,147+offset);
        glVertex2i(pixel_counter,157+offset);
        glEnd();
        sprintf(heading_string,"%3.3d",temp_heading);
        glPushMatrix();
        glTranslatef(pixel_counter-13.0f,162.0f+offset,0.0f);
        glLineWidth( 2 );
        tPrintStrokedString(heading_string);
        glLineWidth( 1 );
        glPopMatrix();
      }
    
      if(heading_check == 2 || heading_check == 4 ||
        heading_check == 6 || heading_check == 8)
      {
        glBegin( GL_LINE_STRIP );
        glVertex2i(pixel_counter,147+offset);
        glVertex2i(pixel_counter,152+offset);
        glEnd();
      }
      temp_heading--;
      pixel_counter-=5;
      loop++;
    }
  }

};



#endif //__SAMPLEDISPLAY_H_
