// SampleEveryDataPoint.h : Declaration of the CSampleEveryDataPoint

#ifndef __SAMPLEEVERYDATAPOINT_H_
#define __SAMPLEEVERYDATAPOINT_H_

#include "resource.h"       // main symbols
#include <atlctl.h>

#include "Iads.h"
#include "IadsTime.h"


#include <math.h>

// OpenGL include files..
#include "gl/gl.h"
#include "gl/glaux.h"  

// Ogl helper functions..
#include "OpenGLHelperTemplates.h"

// OpenGL libraries to link
#pragma comment (lib, "opengl32.lib")
#pragma comment (lib, "glu32.lib")
//#pragma comment (lib, "glaux.lib") Could use this if needed..

typedef struct
{
  unsigned __int16 sync;
  unsigned __int16 sysId;
  unsigned __int16 type;
  unsigned __int16 count;
  unsigned __int16 flags;
  unsigned __int16 checksum1;
  unsigned __int16 source;
  unsigned __int16 beacon;
  unsigned __int16 geoid;
  unsigned __int16 status;
  __int32 u;         // Position X on geoid
  __int32 v;         // Position Y
  __int32 w;         // Position Z
  __int32 psigma;    // Positional Standard Deviation?
  __int32 uv;        // Velocity vector X
  __int32 vv;        // Velocity vector Y
  __int32 wv;        // Velocity vector Z
  __int32 ua;        // Acceleration vector X
  __int32 va;        // Acceleration vector Y
  __int32 wa;        // Acceleration vector Z
  __int32 roll;      // Roll  (Fixed point 1E-6)
  __int32 rollv;     // Roll Velocity vector magnitude
  __int32 pitch;     // Pitch  (Fixed point 1E-6)
  __int32 pitchv;    // Pitch veloctiy vector magnitude
  __int32 heading;   // Heading (Fixed point 1E-6)
  __int32 headingv;  // Heading velocity vector magnitude
  unsigned __int32 seconds;
  unsigned __int16 msec;
  unsigned __int16 checksum2;

} tTECCS_DATA;

const int cNumTeccsStructElements = 29;
static const CComBSTR cTeccsNames[cNumTeccsStructElements] = 
{
  "sync",
  "sysId",
  "type",
  "count",
  "flags",
  "checksum1",
  "source",
  "beacon",
  "geoid",
  "status",
  "u",         // Position X on geoid
  "v",         // Position Y
  "w",         // Position Z
  "psigma",    // Positional Standard Deviation?
  "uv",        // Velocity vector X
  "vv",        // Velocity vector Y
  "wv",        // Velocity vector Z
  "ua",        // Acceleration vector X
  "va",        // Acceleration vector Y
  "wa",        // Acceleration vector Z
  "roll",      // Roll  (Fixed point 1E-6)
  "rollv",     // Roll Velocity vector magnitude
  "pitch",     // Pitch  (Fixed point 1E-6)
  "pitchv",    // Pitch veloctiy vector magnitude
  "heading",   // Heading (Fixed point 1E-6)
  "headingv",  // Heading velocity vector magnitude
  "seconds",
  "msec"
};

/////////////////////////////////////////////////////////////////////////////
// CSampleEveryDataPoint
class ATL_NO_VTABLE CSampleEveryDataPoint : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<ISampleEveryDataPoint, &IID_ISampleEveryDataPoint, &LIBID_SAMPLEEVERYDATAPOINTDISPLAYLib>,
	public CComControl<CSampleEveryDataPoint>,
	public IPersistStreamInitImpl<CSampleEveryDataPoint>,
	public IOleControlImpl<CSampleEveryDataPoint>,
	public IOleObjectImpl<CSampleEveryDataPoint>,
	public IOleInPlaceActiveObjectImpl<CSampleEveryDataPoint>,
	public IViewObjectExImpl<CSampleEveryDataPoint>,
	public IOleInPlaceObjectWindowlessImpl<CSampleEveryDataPoint>,
	public IPersistStorageImpl<CSampleEveryDataPoint>,
	public ISpecifyPropertyPagesImpl<CSampleEveryDataPoint>,
	public IQuickActivateImpl<CSampleEveryDataPoint>,
	public IDataObjectImpl<CSampleEveryDataPoint>,
	public IProvideClassInfo2Impl<&CLSID_SampleEveryDataPoint, NULL, &LIBID_SAMPLEEVERYDATAPOINTDISPLAYLib>,
	public CComCoClass<CSampleEveryDataPoint, &CLSID_SampleEveryDataPoint>,
  public IPersistPropertyBagImpl<CSampleEveryDataPoint>, // Implement this to save in IADS
  public IAcceptIadsParameter,  // Implement this to accept a parameter for every data sample
  public ISynchronizeOutput // Implement this to get time and be driven my IADS with all of the other displays (like stripchart)
{
public:
	CSampleEveryDataPoint()
	{
    // For an OpenGL display, we must be "windowed only". 
    // The window will be used to create an OpenGL drawing context later in "OnCreate" (WM_CREATE handler)
    m_bWindowOnly = TRUE;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SAMPLEEVERYDATAPOINT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSampleEveryDataPoint)
	COM_INTERFACE_ENTRY(ISampleEveryDataPoint)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
  COM_INTERFACE_ENTRY(IPersistPropertyBag)   // <- Don't forget these here.. or the QueryInterface will fail
	COM_INTERFACE_ENTRY(IAcceptIadsParameter)  // <- 
  COM_INTERFACE_ENTRY(ISynchronizeOutput)    // <- 
END_COM_MAP()

BEGIN_PROP_MAP(CSampleEveryDataPoint)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CSampleEveryDataPoint)
	CHAIN_MSG_MAP(CComControl<CSampleEveryDataPoint>)
	DEFAULT_REFLECTION_HANDLER()

  MESSAGE_HANDLER(WM_CREATE,      OnCreate)       // Need this to create our OpenGL drawing context.. Add by hand or wizard
  MESSAGE_HANDLER(WM_DESTROY,     OnDestroy)      // Need this to destroy our OpenGL drawing context.. Add by hand or wizard
  MESSAGE_HANDLER(WM_ERASEBKGND,  OnEraseBkgnd)   // Need this to eliminate redundant background erasing...prevents windows generated flicker
  MESSAGE_HANDLER(WM_SIZE,        OnSize)         // Need this to ensure "glViewport" is set properly on resize

END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// ISampleEveryDataPoint
public:

  STDMETHOD(FinalConstruct)( void )
  {
    // Add constructor code here..

    // OpenGL and Windows stuff..
    mGraphicsContextId = NULL;
    mHDC = NULL;

    mTime = NULL;
    mLastRenderSceneTime = NULL;
    mIsRealTime = VARIANT_TRUE;
    memset( mTeccsData, 0, sizeof(double)*cNumTeccsStructElements );
    memset( mParameters, 0, sizeof(double)*cNumTeccsStructElements );

    return S_OK;
  }

  STDMETHOD(FinalRelease)( void )
  {
    // Add additional destruction code here..
    return S_OK;
  }


  // This is the "Heart" of the drawing code... Put all drawing code here...
  HRESULT OnDraw( ATL_DRAWINFO& di )
	{
    // If the OpenGL context is not initialized yet, don't draw or it may crash...
    if ( !mGraphicsContextId )
    {
      return S_OK;
    }

    // Make our OpenGL "context" the current one (Must do this)
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    // Clear out the background 
    glClear( GL_COLOR_BUFFER_BIT );


    /********** REPLACE WITH YOUR OWN RENDERING CODE HERE ********/
    glLineWidth( 2 );

    char str[1000];
    char strTemp[1000];
    float yPos = 250.0f;
    
    // Print out our data results... Do you own OGL rendering here..
    for ( register int j = 0; j<cNumTeccsStructElements; j++ )
    {
      if ( cTeccsNames[j].m_str ) wcstombs( strTemp, cTeccsNames[j], cTeccsNames[j].Length()+1 );
      else strcpy( strTemp, "" );
      sprintf( str, "%d", (int)mTeccsData[j] );
      glPushMatrix();
        glTranslatef( 100.0, yPos, 0.0 );
        tPrintStrokedString( str );
        yPos -= 10.0f;
      glPopMatrix();
    }
    /*************************************************************/


    // Output our rendering to the screen
    ::SwapBuffers( mHDC );
    return S_OK;
	}

  // IAcceptIadsParameter: IADS specific interface to allow "Parameter" object to be connected to displays..
  // JFB: This interface may not be needed if "IParameter" objects are allowed in properties, and the container handles it..
  STDMETHODIMP get_AttachPointCount( /*[out,retval]*/ long* pVal )
  {
    if ( !pVal ) return E_POINTER;
    *pVal = cNumTeccsStructElements;  // 29 Teccs elements in the struct... So we want 29 "AttachPoints".. Each struct element will be a seperate "parameter"
    return S_OK;
  }

  // Caller will give us number, we need to return Ascii name for that index. These names will be shown on the 
  // popup menu after parameter is dropped...
  STDMETHODIMP AttachPointName( /*[in]*/ long index, /*[out,retval]*/BSTR* ppItem )
  {
    if ( !ppItem ) return E_POINTER;
    if ( index < 0 || index >= cNumTeccsStructElements ) return E_INVALIDARG;  
    *ppItem = cTeccsNames[index].Copy();  // return the string element from the list above..
    return *ppItem ? S_OK : E_OUTOFMEMORY;
  }

  STDMETHODIMP get_AttachPointDepth( /*[in]*/ long index, /*[out,retval]*/long* pVal )
  {
    if ( !pVal ) return E_POINTER;
    *pVal = 1;  // Only one parameter (no parameter "pairs" needed in this display)
    return S_OK;
  }


  STDMETHODIMP get_Count( /*[out, retval]*/ long* pVal )
  {
    if ( !pVal ) return E_POINTER;
    *pVal = cNumTeccsStructElements; // How many parameters attached? Logic Error!!! Return same value get_AttachPointCount
    return S_OK;
  }

  STDMETHODIMP get__NewEnum( /*[out, retval]*/ IUnknown** ppVal )
  {
    return E_NOTIMPL;  // Later for VB clients
  }

  STDMETHODIMP Item( /*[in]*/ VARIANT AttachPointNameOrIndex, /*[in,optional,defaultvalue(0)]*/ long AttachPointDepthIndex, /*[out,retval]*/ IParameter** ppItem )
  {
    if ( AttachPointNameOrIndex.vt != VT_I4 ) return E_INVALIDARG; // Fix later...Only allow integer index for now.. Handle strings later..

    LONG index = AttachPointNameOrIndex.lVal;

    // check if in range of mParameters here... return E_INVALIDARG; if out of bounds..
    if ( index < 0 || index >= cNumTeccsStructElements ) return E_INVALIDARG;  

    *ppItem = mParameters[index];
    if ( mParameters[index] ) mParameters[index]->AddRef();  // Add a ref count so it doesn't get deleted

    return S_OK;
  }

  STDMETHODIMP Remove( /*[in]*/ VARIANT AttachPointNameOrIndexOrObject, /*[in,optional,defaultvalue(0)]*/long AttachPointDepthIndex )  // ?? by [in] VARIANT indexOrName???
  {
    return E_NOTIMPL;  // Later
  }
  STDMETHODIMP RemoveAll()
  {
    return E_NOTIMPL; // Later
  }

  // IADS will call this function when a user drops a parameter on your display...
  STDMETHODIMP Add( /*[in]*/ IParameter* newVal, /*[in,optional,defaultvalue("(null)")]*/ BSTR AttachPointName, /*[in,optional,defaultvalue(0)]*/long AttachPointDepthIndex )
  {
    if ( !newVal ) return E_POINTER;

    // Get the index from the AttachPointName
    for( register int i=0; i<cNumTeccsStructElements; i++ )
    {
      if ( wcscmp(cTeccsNames[i],AttachPointName) == 0 )
        break;
    }
    if ( i >= cNumTeccsStructElements ) return E_INVALIDARG; // Didn't find the name passed in...

    // Release old parameter, if exists..
    if ( mParameters[i] ) mParameters[i]->Release();

    mParameters[i] = newVal;  // Grab new parameter
    newVal->AddRef();         // Add a ref..

    this->FireViewChange();   // Force a redraw of our display

    m_bRequiresSave = TRUE;   // Dirty our display so it'll get saved out...
    return S_OK; 
  }


  /////////// ISynchronizeOutput: IADS specific interface to synchronize drawing and data output with other displays on the screen
  STDMETHODIMP PrepareOutput( /*[in]*/ IIadsTime* parentContainerTime, /*[in]*/ IIadsTime* globalTime )
  {
    if ( !parentContainerTime || !globalTime ) return E_POINTER;
    if ( !mTime ) ::CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_ALL, IID_IIadsTime, (void**)&mTime );
    if ( !mLastRenderSceneTime ) ::CoCreateInstance( CLSID_IadsTime, NULL, CLSCTX_ALL, IID_IIadsTime, (void**)&mLastRenderSceneTime );
    if ( !mTime || !mLastRenderSceneTime ) return E_UNEXPECTED;

    if ( this->mTime->SetTime(parentContainerTime) != S_OK ) return E_UNEXPECTED;

    // Check to see if time is invalid... If time was invalid just do nothing until good...
    __int64 t1 = 0;
    __int64 t2 = 0;
    mLastRenderSceneTime->get_TotalNanoseconds( &t1 );
    mTime->get_TotalNanoseconds( &t2 );
    if ( t1 < 0I64 || t2 < 0I64 )  // If time is negative, it's invalid
    {
      mLastRenderSceneTime->SetTime( mTime );
      return S_OK; 
    }

    // Now, depending on the "mode" we are in, treat time differently... see "put_RealTime" for explanation
    if ( mIsRealTime )
    {
      // Check for time jump too great...
      int numSecs;
      mLastRenderSceneTime->TimeTicksBetween( mTime, 1.0, &numSecs );
      if ( numSecs > 5 || numSecs < 0 ) 
      { 
        mLastRenderSceneTime->SetTime( mTime ); 
        if ( numSecs > 5 )
          ::OutputDebugString( "Time jumped by more than 5 seconds\n" );
        else
          ::OutputDebugString( "Time wrapped, resyncing\n" );
        return S_OK; 
      }
    }
    else   // Scrollback mode!! User is changing time back and forth via the scrollbar
    {
      // IF you want to recompute every time the user scrolls... Determine the amount of time and set in the Retract time..
      mLastRenderSceneTime->SetTime( mTime );
      mLastRenderSceneTime->RetractTime( 10, 1.0 );  // Retract by 10 seconds and recompute total forward
    }

    // To make things easy on me, I'll assume that *every* parameter has the same sample rate, and that I won't do any work
    // unless a parameter is attached to my first "AttachPoint"
    // If all of parameters are not the same sample rate (they will be in a TECCS packet/blob case, then see "IterationSampleRate" in IParameter and make them all the same...
    if ( !mParameters[0] ) return S_OK;

    //int numSecs;
    //mLastRenderSceneTime->TimeTicksBetween( mTime, 1.0, &numSecs );
    //if ( numSecs < 3 ) 
    //  return S_OK;

    // Now, go to the *last* time that we finished processing... and start there... and stop at the "current" time
    HRESULT hr = mParameters[0]->put_CurrentTime( mLastRenderSceneTime );
    if ( hr != S_OK ) { mLastRenderSceneTime->SetTime(mTime); return S_OK; }

    CComVariant v;
    long numPts;
    hr = mParameters[0]->GetNumDataPoints( mLastRenderSceneTime, mTime, &numPts );    

    // Get the block of data
    for ( register int j = 0; j<cNumTeccsStructElements; j++ )
    {
      if ( !mParameters[j] ) continue;

      // Keep the predictive cache rolling.. always go to the current time before doing anything else
	    hr = mParameters[j]->put_CurrentTime( mTime ); 

      // Now, go back to data
	    hr = mParameters[j]->put_CurrentTime( mLastRenderSceneTime );

		  hr = mParameters[j]->Values( numPts, iadsTimeNoChange, &v );

      // We've already checked the first point of the array last time..
      SAFEARRAY* sa = v.parray;
      double* d = (double*)sa->pvData;
      int ub = cNumTeccsStructElements > sa->rgsabound->cElements ? sa->rgsabound->cElements : cNumTeccsStructElements;
      for ( register int k=0; k<ub; k++ )
      {
        mTeccsData[k] = d[k]; 
      }
      for ( k=ub; k<cNumTeccsStructElements; k++ )
      {
        mTeccsData[k] = 0.0; 
      }
    }


    // We have something new to draw now, so invalidate our display area (will cause our "OnDraw" method to be called in due time)
    this->FireViewChange(); 

    // Keep track of our last time we calculated data, so we can ensure that we don't miss any data points when we "wake up" next "PrepareOutput"
    mLastRenderSceneTime->SetTime( mTime );
    
    return S_OK;
  }

  STDMETHODIMP PerformOutput()
  {
    // If our display was "DoubleBuffered", we would do our final "PageFlip/Bitblit" output here....
    // We really don't need method call for this particular display (Unless we are windowed/double buffered)
    // Do all of the drawing in OnDraw..
    return S_OK;
  }

  STDMETHODIMP put_IsRealTime( /*[in]*/ VARIANT_BOOL isRealTime )
  {
    if ( mIsRealTime == isRealTime ) return S_OK;

    // RealTime state changed here... Need to do anything extra???
    // "RealTime = VARIANT_TRUE means that the user is running the display continuously updating 
    // from the IADS clock
    // "RealTime = VARIANT_FALSE means that the user is paused or is in "scrollback" mode, updating
    // from the analysis window's scrollbar position

    mIsRealTime = isRealTime;
    return S_OK;
  }
  //////////////////////





  // The following methods are "Event Handlers" for windows events. In order to have the methods called, we must first
  // add a handler to the "BEGIN_MSG_MAP" area of the code above. You can do this using the wizard or by hand..
  
  // This method gets called after our "Window" is created... At this time, it's ok to create the OpenGL drawing context
  virtual LRESULT OnCreate( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
  {
    // Get our "DC" (DeviceContext), we'll need this to "SwapBuffers" later...
    mHDC = this->GetDC();

    BOOL isHardware = TRUE;    // Always default to hardware unless shut off...
    BOOL verticalSync = FALSE; // Wait for vertical sync? No.. It runs faster and we have seen no "tearing" so far..
    BOOL doubleBuffer = TRUE;  // Always default to double buffered to reduce tearing

    // Call the helper functions to setup the OpenGL drawing context
    tSetupOGLPixelFormat( mHDC, isHardware, doubleBuffer, verticalSync, 16, 32, &mPfd );

    mGraphicsContextId = wglCreateContext( mHDC );
    if ( mGraphicsContextId == NULL ) return E_FAIL;
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    // Create call lists so we can output text...Don't use this example for your code... See OpenGlHelperTemplates.h
    tGenerateNumberCallLists();

    // Set the clear color
    glClearColor( 0.70f, 0.70f, 0.70f, 1.0f );  

    // Set the ortho... Doesn't change for this display
    glOrtho( 0.0, 270.0, 0.0, 270.0, -1.0,  1.0 );

    return S_OK;
  }

  // This method gets called just before our "Window" is being destroyed... Clean up the OpenGL drawing context here..
  virtual LRESULT OnDestroy( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled )
  {
    ::wglMakeCurrent( NULL, NULL );  // Don't forget this!! Must make sure that we're not the current context before destruction
  
    ReleaseDC( mHDC );  // Release our DC.. Windows cleanup..
  
    // Delete our OpenGL drawing context..
    if ( mGraphicsContextId )
    {
      ::wglDeleteContext( mGraphicsContextId );
      mGraphicsContextId = NULL;
    }

    return 0;  // Let windows do its final cleanup
  }

  // Flicker prevention..
  virtual LRESULT OnEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
  {
    // By returning 1 and "bHandled=TRUE" we are telling windows NOT to erase our background. This will prevent
    // a redundant "EraseBackground" operation that will produce an obnoxious "flicker" on the screen. 
    // We'll let the OpenGL window do all erasing/drawing from now on.... The "WM_PAINT" message will eventually
    // call our "OnDraw" method above. That's were all of the OpenGL rending code will be called..
    bHandled = TRUE;
    return 1;
  }

    // Set the OpenGL viewport properly every time someone resizes us...
  virtual LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
  {
    if ( !mGraphicsContextId || !mHDC ) return 0;

    RECT rc;
    GetClientRect( &rc );
    ::wglMakeCurrent( mHDC, mGraphicsContextId );

    ::glViewport( 0,0, rc.right - rc.left, rc.bottom - rc.top );   
  
    this->FireViewChange();
    return 0;
  }


  HDC                   mHDC;
  HGLRC                 mGraphicsContextId; 
  PIXELFORMATDESCRIPTOR mPfd;

  IParameter*           mParameters[cNumTeccsStructElements];       // Allow 29 different "parameters" to be attached to this display (slots in DV), each corresponding to an element in a blob/packet/struct
  IIadsTime*            mTime;                // Current time..
  IIadsTime*            mLastRenderSceneTime; // Last time that we got data and processed it...
  VARIANT_BOOL          mIsRealTime;          // Is the user scrolling (with scrollbar) or streaming...

  tTECCS_DATA           mCurrentTeccsPacket;  // Our data storage area... I'm too lazy to set these struct values so I'll use the array below..
  double                mTeccsData[cNumTeccsStructElements]; // Used to hold a running sum (every data point) elements of the struct above.. I'm too lazy to set individual structure elements..

};

#endif //__SAMPLEEVERYDATAPOINT_H_
